/*
 * Processor.java
 *
 * Copyright (C) 2011 John Douyere (VK2ETA)
 * Based on Pskmail from Per Crusefalk and Rein Couperus
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.AndFlmsg;

import java.util.Calendar;
import java.util.concurrent.Semaphore;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.*;

import android.app.Notification;
import android.app.Service;
import android.content.Intent;
import android.os.Environment;
import android.os.IBinder;

/**
 *
 * @author John Douyere (VK2ETA) 
 * 
 */
public class Processor extends Service{

	static String application ="AndFlmsg A0.2"; // Used to preset an empty status 
	static String version = "Version A0.2.0, 2014-12-21";

	static boolean onWindows = true;
	static String ModemPreamble="";  // String to send before any Tx Buffer
	static String ModemPostamble=""; // String to send after any Tx buffer
	static String HomePath = "";
	static String Dirprefix = "";
	static final String DirInbox = "Inbox";
	static final String DirDrafts = "Drafts";
	static final String DirOutbox = "Outbox";
	static final String DirSent = "Sent";
	static final String DirEntryForms = "EntryForms";
	static final String DirDisplayForms = "DisplayForms";
	static final String DirTemplates = "Templates";
	static final String DirTemp = "Temp";
	static final String DirLogs = "Logs";
	static final String messageLogFile = "messagelog.txt";
	static String Separator = "";
	static boolean compressedMsg = false;
	static String Sendline = "";
	static String SendCommand="";
	static String CrcString = "";
	static String FileNameString = "";
	static int DCD  = 0;
	static int MAXDCD = 3;
	static boolean  TXActive = false;
	
	//JD temp FIX: init as first modem in list
	static int TxModem = Modem.customModeListInt[0];
	static int RxModem = Modem.customModeListInt[0];;

	//Semaphores to instruct the RxTx Thread to start or stop
	public static Semaphore restartRxModem = new Semaphore(1, false);

	// globals to pass info to gui windows
	static String monitor = "";
	static String TXmonitor = "";
	static String TermWindow = "";
	static String Status = "Listening";
	static int cpuload;

	// globals for communication
	static  String mycall;     // my call sign from options
	static int DCDthrow;
	static boolean ReceivingForm = false;
	static String TX_Text; // output queue
	// Error handling and logging object
	static loggingclass log;


	public static void processor() {
		//Nothing as this is a service
	}

	@Override
	public void onCreate() {

    	//Save Environment if we need to access Java code/variables from C++
    	Message.saveEnv(); 
    	
    	// Create error handling class
		log = new loggingclass("AndFlmsg");

		// Get settings and initialize
		handleinitialization();

		//Initialize Modem (creates the various type of modem objects)
		Modem.ModemInit();

		//Check that we have a current mode, otherwise take the first one in the list (useful when we have a NIL list of custom modes)
		Processor.RxModem = Processor.TxModem = Modem.customModeListInt[Modem.getModeIndex(Processor.RxModem)];

		//Reset frequency and squelch
		Modem.reset();

		Modem.send_rsid_command("ON");

		//Make sure the display strings are blank
		Processor.monitor = "";
		Processor.TXmonitor = "";
		Processor.TermWindow = "";
	}


	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
			// Start the RxThread
			Modem.startmodem();
			
			// Use a ticker to notify user of the modem running in the background
			CharSequence text = getText(R.string.Modem_Running);

			// Set the scrolling text
			Notification notification = new Notification(0, text, 0);

			//JD make sure Android keeps this running even if resources are restrained
			startForeground(1, notification);

		// Keep this service running until it is explicitly stopped, so use sticky.
		return START_NOT_STICKY;
	}
	
	
	@Override
	public void onDestroy() {
		// Kill the Rx Modem thread
			Modem.stopRxModem();
	}  



	//Post to main terminal window
	public static void PostToTerminal(String text) {
		Processor.TermWindow += text;
		AndFlmsg.mHandler.post(AndFlmsg.addtoterminal);
	}


	//Post to main terminal window
	public static void PostToModem(String text) {
		Processor.monitor += text;
		AndFlmsg.mHandler.post(AndFlmsg.addtomodem);
	}


	//Process one block of received data. Called from Modem.RxBlock when a complete 
	//  block is received
	public static void processWrapBlock(String Blockline) {
		FileWriter out = null;
		try {
			boolean unwrapResult = Message.ProcessWrapBuffer(Blockline);
			if (unwrapResult) {
				//Save file and log it
				String resultString = Message.getUnwrapText();
				String resultFilename = Message.getUnwrapFilename();
				File msgReceivedFile = new File(Processor.HomePath + 
						  Processor.Dirprefix + Processor.DirInbox + Processor.Separator + resultFilename);
						if (msgReceivedFile.exists()) {
							msgReceivedFile.delete();
						}
				out = new FileWriter(msgReceivedFile, true);
				out.write(resultString);
				out.close();
				PostToTerminal("\nFile integrity ckeck: Checksum OK");
				PostToTerminal("\nSaved File: " + resultFilename);
				Message.addEntryToLog("Received Message " + resultFilename);
			} else {
				PostToTerminal("\n " + Message.geterrtext());
				//JD To-Do: add saving as text file here in case of corrupt data
			}
		} catch (Exception ex) {
			Logger.getLogger(Processor.class.getName()).log(Level.SEVERE, null, ex);

		} 
	}


	/**
	 * Create or check the necessary folder structure 
	 */
	public static void handlefolderstructure(){
		
		try {
			//VK2ETA To-Do: add exception here when there is no external storage
			HomePath = Environment.getExternalStorageDirectory().getAbsolutePath();
			// are we on Linux?
			if (File.separator.equals("/")) {
				Separator = "/";
				Dirprefix = "/NBEMS.files/";
				onWindows = false;
			} else  {
				Separator = "\\";
				Dirprefix = "\\NBEMS.files\\";
				onWindows = true;
			}

			//Check if base directory exists, create if not
			File baseDir = new File(HomePath + Dirprefix);
			if (!baseDir.isDirectory()) {
				baseDir.mkdir();
			}

			File thisDir;
			
			//Check if Inbox directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirInbox + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Outbox directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirOutbox + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Sent directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirSent + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Data Entry Forms directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirEntryForms + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Display Forms directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirDisplayForms + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Templates directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirTemplates + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Drafts directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirDrafts + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Logs directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirLogs + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			
			//Check if Temp directory exists, create if not
			thisDir = new File(HomePath + Dirprefix + DirTemp + Separator);
			if (!thisDir.isDirectory()) {
				thisDir.mkdir();
			}
			// Now clears the Temp directory of temporary files
			// This filter only returns files
			FileFilter fileFilter = new FileFilter() {
				public boolean accept(File file)
				{
					return file.isFile();
				}
			};
			//Build array of strings containing the file names
			File[] files = thisDir.listFiles(fileFilter);
			for (int i = 0; i < files.length; i++)
			{
				files[i].delete();
			}
			
		}
		catch(Exception ex){
			loggingclass.writelog("Problem when handling AndFlmsg folder structure.", ex, true);
		}
	}

	private static void handleinitialization(){

		try {
			// Initialize send queue
			TX_Text = "";
			ModemPreamble = config.getPreferenceS("MODEMPREAMBLE", "");
			ModemPostamble = config.getPreferenceS("MODEMPOSTAMBLE", "");
			// Compression settings
			compressedMsg = config.getPreferenceB("COMPRESSED");
			Processor.mycall = config.getPreferenceS("CALL");
		} catch (Exception e) {
			loggingclass.writelog("Problems with config parameter.", e, true);
		}
	}


	/* not used

	static void savePreferences()  {
		try {
			// store the config file if present

			File f1 = new File( "configuration.xml");
			File f2 = new File(Processor.HomePath + Processor.Dirprefix + "configuration.xml");

			if (f1.isFile()) {

				InputStream in = new FileInputStream(f1);

				//Overwrite the file.
				OutputStream out = new FileOutputStream(f2);

				byte[] buf = new byte[1024];
				int len;
				while ((len = in.read(buf)) > 0){
					out.write(buf, 0, len);
				}
				in.close();
				out.close();
				q.Message("Config File stored.", 10);
			}
		}
		catch (Exception e) {
			q.Message("problem writing the config file", 10);
		}
	}

*/

	static double decayaverage(double oldaverage, double newvalue, double factor)  {

		double newaverage = oldaverage;
		if (factor > 1) {
			newaverage = (oldaverage * (1 - 1 / factor)) + (newvalue / factor);
		}
		return newaverage;
	}

	//Return current time as a String
	static String myTime () {
		// create a java calendar instance
		Calendar calendar = Calendar.getInstance();

		// get a java.util.Date from the calendar instance.
		// this date will represent the current instant, or "now".
		java.util.Date now = calendar.getTime();

		// a java current time (now) instance
		java.sql.Timestamp currentTimestamp = new java.sql.Timestamp(now.getTime());

		return currentTimestamp.toString().substring(0,16);
	}

	//Logging
	static void log (String logtext) {
		//           File consolelog = new File (HomePath + Dirprefix + "logfile");
		try{
			// Create file
			FileWriter logstream = new FileWriter(HomePath + Dirprefix + "logfile", true);
			BufferedWriter out = new BufferedWriter(logstream);

			out.write(myTime() + " " + logtext + "\n");
			//Close the output stream
			out.close();

		}catch (Exception e){//Catch exception if any
			//              System.err.println("LogError: " + e.getMessage());
			loggingclass.writelog("LogError " + e.getMessage() , null, true);
		}
		Processor.PostToTerminal(myTime() + " " + logtext + "\n");
	}


	@Override
	public IBinder onBind(Intent arg0) {
		// Nothing here, not used
		return null;
	}


}

